Imports System.Math

Module GameGraphics
    Private CurImg As Integer = 1
    Private MainImg As PictureBox
    Private Background As New MBmp(1280, 960)
    Private Img720 As New MBmp(1280, 720)
    Private DisplayMode As Integer
    Private Width As Integer
    Private Height As Integer
    Private MainForm As Form


#Region "Init"
    Public Sub SetGraphics(ByRef PicBox As PictureBox, ByRef MForm As Form)
        MainImg = PicBox
        MainForm = MForm
    End Sub

#End Region

#Region "Background Functions"

    Public Sub SetBackground(ByVal Img As Bitmap)
        'convert a bitmap to mbmp
        Dim TMBMP() As Byte = Background.GetBMPDataRef
        Dim R, G, B As Byte
        Dim TColor As Color
        If Img.Width <> Width Or Img.Height <> Height Then
            Exit Sub
        End If
        If Img.PixelFormat = Imaging.PixelFormat.Format24bppRgb And Img.RawFormat.GetHashCode = -1102893738 Then
            LockBitmap(Img, MemRead)
            Background.SetBMPData(g_PixBytes)
            UnlockBitmap(Img, MemWrite)
        Else
            Dim X As Integer : Dim Y As Integer
            Dim YIndex As Integer : Dim XIndex As Integer
            For Y = 0 To Img.Height - 1
                YIndex = 3 * Img.Width * Y
                For X = 0 To Img.Width - 1
                    XIndex = YIndex + (X * 3)
                    TColor = Img.GetPixel(X, Y)
                    R = TColor.R
                    G = TColor.G
                    B = TColor.B
                    TMBMP(XIndex) = B
                    TMBMP(XIndex + 1) = G
                    TMBMP(XIndex + 2) = R
                Next
            Next
        End If
    End Sub

    Public Sub SetBackground(ByVal Img As MBmp)
        Background = Img.Clone
    End Sub

    Public Function GetBackground()
        GetBackground = Background.Clone
    End Function

#End Region

    Public Sub SetDisplayMode(ByVal Mode As Integer)
        'Use 4/3 - Mode 1 1280x960, Mode 2(shrink for 800x600)
        'Use 5/4 - Mode 3 1280x1024 (use 1280x960 with black bars on top and bottom)
        'Use 16/9 - Mode 4 1280x720 Mode 5(upconvert for 1920x1080)
        If Mode = 1 Then
            DisplayMode = 1
            Width = 1280
            Height = 960
            MainImg.Width = 1280
            MainImg.Height = 960
            MainForm.Width = 1288
            MainForm.Height = 1004
        ElseIf Mode = 2 Then
            DisplayMode = 2
            Width = 1280
            Height = 960
            MainImg.Width = 800
            MainImg.Height = 600
            MainForm.Width = 808
            MainForm.Height = 644
        ElseIf Mode = 3 Then
            DisplayMode = 3
            Width = 1280
            Height = 960
            MainImg.Width = 1280
            MainImg.Height = 1024
            MainForm.Width = 1288
            MainForm.Height = 1068
        ElseIf Mode = 4 Then
            DisplayMode = 4
            Width = 1280
            Height = 720
            MainImg.Width = 1280
            MainImg.Height = 720
            MainForm.Width = 1288
            MainForm.Height = 764
        ElseIf Mode = 5 Then
            DisplayMode = 5
            Width = 1280
            Height = 720
            MainImg.Width = 1920
            MainImg.Height = 1080
            MainForm.Width = 1928
            MainForm.Height = 1124
        End If
        Background = New MBmp(Width, Height)
        MainImg.Refresh()
    End Sub

    Public Function Display(ByRef Screen As MBmp)
        'Set Preferances
        'Use 4/3 - Mode 1 1280x960, Mode 2(shrink for 800x600)
        'Use 5/4 - Mode 3 1280x1024 (use 1280x960 with black bars on top and bottom)
        'Use 16/9 - Mode 4 1280x720 Mode 5(upconvert for 1920x1080) (use 1280x960 and cut off top 240)
        If Screen.GetWidth <> 1280 Then
            Display = 1 'wrong size
            Exit Function
        Else
            If DisplayMode >= 1 And DisplayMode <= 3 Then
                If Screen.GetHeight <> 960 Then
                    Display = 1 'wrong size
                    Exit Function
                Else
                    If DisplayMode = 3 Then
                        Dim Tmbmp As New MBmp(1280, 1024)
                        Array.Copy(Screen.GetBMPData, 0, Tmbmp.GetBMPDataRef, 122880, Screen.GetDataLen)
                        'MainImg.Image = Tmbmp.ConvertToBitmap()
                        LockBitmap(MainImg.Image, MemWrite)
                        g_PixBytes = Tmbmp.GetBMPDataRef
                        UnlockBitmap(MainImg.Image, MemWrite)
                    Else
                        LockBitmap(MainImg.Image, MemWrite)
                        g_PixBytes = Screen.GetBMPDataRef
                        UnlockBitmap(MainImg.Image, MemWrite)
                    End If
                End If
            ElseIf DisplayMode = 4 Or DisplayMode = 5 Then
                If Screen.GetHeight = 720 Then
                    MainImg.Image = Screen.ConvertToBitmap()
                ElseIf Screen.GetHeight = 960 Then
                    Array.Copy(Screen.GetBMPDataRef, 921600, Img720.GetBMPDataRef, 0, 2764800)
                    MainImg.Image = Img720.ConvertToBitmap
                Else
                    Display = 1 'wrong size
                    Exit Function
                End If
            End If
        End If
        MainImg.Refresh()
    End Function

    Public Function GetCurrentFrame()
        Dim BaseMbmp As New MBmp(MainImg.Image)
        GetCurrentFrame = BaseMbmp.Clone
        BaseMbmp = Nothing
    End Function

    Public Sub Earthquake(ByVal Magnitude As Integer, ByVal Duration As Double)
        Dim BaseMbmp As MBmp = GetCurrentFrame()
        Dim DispMbmp As MBmp = BaseMbmp.Clone
        Dim TBaseMbmp As New MBmp(BaseMbmp.GetWidth, BaseMbmp.GetHeight + (2 * Magnitude))
        Dim DataLength As Integer = BaseMbmp.GetDataLen
        Dim RowsChange, ArrayOffset As Integer
        Dim ZeroOffset As Integer = Magnitude * BaseMbmp.GetRowWidth
        Dim I As Integer = 0
        Dim EndTime As Double
        Array.Copy(BaseMbmp.GetBMPDataRef, 0, TBaseMbmp.GetBMPDataRef, Magnitude * BaseMbmp.GetRowWidth, DataLength)
        BaseMbmp = TBaseMbmp.Clone
        TBaseMbmp = Nothing

        EndTime = GetTimer() + Duration
        Do
            I = I + 45
            'create dispmbmp
            RowsChange = Round(Magnitude * Sin(I * PI / 180), 0)
            ArrayOffset = (RowsChange * BaseMbmp.GetRowWidth) + ZeroOffset
            Array.Copy(BaseMbmp.GetBMPDataRef, ArrayOffset, dispMbmp.GetBMPDataRef, 0, DataLength)
            Display(dispMbmp)
            'Wait(1 / 60)
        Loop Until GetTimer() > EndTime
        Array.Copy(BaseMbmp.GetBMPDataRef, ZeroOffset, dispMbmp.GetBMPDataRef, 0, DataLength)
        Display(dispMbmp)
        MsgBox(I / 45)
        'Clean up large structures
        BaseMbmp = Nothing
        DispMbmp = Nothing
    End Sub

    Public Function GetNewFrame()
        GetNewFrame = New MBmp(1280, 960)
    End Function

    Public Function GetDisplayMode()
        Return DisplayMode
    End Function

    Public Function GetWidth()
        Return Width
    End Function

    Public Function GetHeight()
        Return Height
    End Function

#Region "Helper Functions"

    Public Function IntegerToBytes(ByVal Value As Integer)
        If Value >= 0 Then
            Dim IntArray(3) As Byte
            IntArray(0) = CByte((&HFF000000 And Value) / 16777216)
            IntArray(1) = CByte((&HFF0000 And Value) / 65536)
            IntArray(2) = CByte((&HFF00 And Value) / 256)
            IntArray(3) = CByte(&HFF And Value)
            IntegerToBytes = IntArray
        End If
    End Function

    Public Function BytesToInteger(ByVal IntArray() As Byte)
        Dim Value As Integer
        If IntArray(0) < 128 Then
            Value = (IntArray(0) * 16777216) + (IntArray(1) * 65536) + (IntArray(2) * 256) + IntArray(3)
            BytesToInteger = Value
        End If
    End Function

#End Region


End Module

Module SpeechBox
    Private Width, Height, RowWidth, VBMPDataLen As Integer
    Private SpeechBoxStart, TxtBoxStart, TxtBoxEnd As Point
    Private Loaded As Boolean = False
    Private BaseImage() As Byte
    Private DisplayImage() As Byte
    Private VBMPData() As Integer 'Location point, Run length

    '******Needs to be tested
    Public Sub LoadBase(ByVal VBMPImage As VBMP)
        Loaded = False
        Width = VBMPImage.GetWidth
        Height = VBMPImage.GetHeight
        RowWidth = VBMPImage.GetRowWidth
        Dim BArray(RowWidth * Height - 1) As Byte 'Do not set to nothing at end
        Dim DArray(RowWidth * Height - 1) As Byte 'Do not set to nothing at end
        Dim TArray() As Byte = VBMPImage.GetVBMPDataRef 'Do not set to nothing at end
        Dim TVBMPDataArray(5000) As Integer
        Dim TVBMPDataArray2() As Integer 'Do not set to nothing at end
        Dim IntArray(3) As Byte
        Dim VBMPDataIndex As Integer = 0
        Dim VBMPDataSegLen As Integer
        Dim CurPosVBMP As Integer = 0
        Dim CurRowVBMP As Integer = 0
        VBMPDataLen = 0

        'Load data into BArray
        Do Until VBMPDataIndex >= VBMPImage.GetDataLen
            'Get to start point
            Array.Copy(TArray, VBMPDataIndex, IntArray, 0, 4)
            CurPosVBMP = BytesToInteger(IntArray)
            Array.Copy(TArray, VBMPDataIndex + 4, IntArray, 0, 4)
            VBMPDataSegLen = BytesToInteger(IntArray)
            VBMPDataIndex = VBMPDataIndex + 8
            Array.Copy(TArray, VBMPDataIndex, BArray, CurPosVBMP, VBMPDataSegLen)
            VBMPDataIndex = VBMPDataIndex + VBMPDataSegLen
            TVBMPDataArray(VBMPDataLen) = CurPosVBMP
            TVBMPDataArray(VBMPDataLen + 1) = VBMPDataSegLen
            VBMPDataLen = VBMPDataLen + 2
        Loop

        ReDim TVBMPDataArray2(VBMPDataLen - 1)
        Array.Copy(TVBMPDataArray, TVBMPDataArray2, VBMPDataLen)
        Array.Copy(BArray, DArray, RowWidth * Height)
        VBMPData = TVBMPDataArray2
        BaseImage = BArray
        DisplayImage = DArray
        Loaded = True
        TVBMPDataArray = Nothing
    End Sub

    'This should only be called once.  It sets the position of the SpeechBox on the main image.
    Public Sub SetPosition(ByVal StartPoint As Point)
        SpeechBoxStart = StartPoint
    End Sub

    'Resets the DisplayImage to the BaseImage
    Public Sub ResetDisplay()
        Array.Copy(BaseImage, DisplayImage, Width * Height * 3)
    End Sub

    'When switching from 960 main image height to a 720
    Public Sub GoTo720()
        SpeechBoxStart.X = SpeechBoxStart.X - 120
    End Sub

    'When switching from 720 main image height to a 960
    Public Sub GoTo960()
        SpeechBoxStart.X = SpeechBoxStart.X + 120
    End Sub

    'Returns if the SpeechBox has been initalized
    Public Function IsLoaded()
        IsLoaded = Loaded
    End Function
End Module